/**
 * @author	WoltLab Community
 * @copyright	2009-2010 WoltLab Community
 * @license	GNU Lesser General Public License <http://www.gnu.org/licenses/lgpl.html>
 */
var PortalEditor = Class.create({
	/**
	 * Initializes the PortalEditor class
	 */
	initialize: function() {
		var checkboxes = $$('.portalEditActivate');
		
		// toggles box activation status including icon and content display
		checkboxes.each(function(checkbox) {
			var boxID = parseInt(checkbox.identify().gsub('activated', ''))
			var container = checkbox.up().down('.container-1');
			var icon = $('activatedIcon'+boxID);
			
			if (container) {
				if (!checkbox.checked) {
					container.hide();
				}
				
				checkbox.observe('change', this.toggleContent.bind(this, checkbox, container, icon));
			}
		}.bind(this));
		
		this.createSortables();
	},
	/**
	 * Toggles content and corresponding icon
	 */
	toggleContent: function(checkbox, container, icon) {
		if (checkbox.checked) {
			container.blindDown({ duration: 0.5 });
			icon.writeAttribute('src', icon.readAttribute('src').gsub('boxDeactivatedS', 'boxActivatedS'));
		}
		else {
			container.blindUp({ duration: 0.5 });
			icon.writeAttribute('src', icon.readAttribute('src').gsub('boxActivatedS', 'boxDeactivatedS'));
		}
	},
	/**
	 * Creates sortable containers and elements
	 */
	createSortables: function() {
		Sortable.create('centerboxes', {
			tag: 'div',
			treeTag: 'div',
			scroll: window,
			onChange: function(element) {
				this.changeBoxParameters(element, 1);
			}.bind(this)
		});
		
		Sortable.create('rightboxes', {
			tag: 'div',
			treeTag: 'div',
			dropOnEmpty: true,
			scroll: window,
			containment: [ 'leftboxes', 'rightboxes' ],
			constraint: null,
			onChange: function(element) {
				this.changeBoxParameters(element, 3);
			}.bind(this)
		});
		
		Sortable.create('leftboxes', {
			tag: 'div',
			treeTag: 'div',
			scroll: window,
			dropOnEmpty: true,
			containment: [ 'leftboxes', 'rightboxes' ],
			constraint: null,
			onChange: function(element) {
				this.changeBoxParameters(element, 2);
			}.bind(this)
		});
	},
	/**
	 * Handles internal sort order
	 */
	changeBoxParameters: function(element, boxTypeID) {
		element.up().childElements().each(function(box, position) {
			var input = box.select('input');
			input[1].value = position + 1;
			
			if (boxTypeID != 1) {
				input[2].value = boxTypeID;
			}
		});
	}
});