<?php
// wbb imports
require_once(WBB_DIR.'lib/acp/action/PortalBoxAction.class.php');
require_once(WCF_DIR.'lib/system/io/TarWriter.class.php');

/**
 * Exports portal boxes
 *
 * @package		com.woltlab.community.portal
 * @author		WoltLab Community
 * @copyright	2009 WoltLab Community
 * @license		GNU Lesser Public License <http://www.gnu.org/licenses/lgpl.html>
 * @subpackage	acp.action
 * @category	Portal
 */
class BoxExportAction extends PortalBoxAction {
	
	/**
	 * Exports boxes
	 *
	 * @return	void
	 * @see		Action::execute()
	 */
	public function execute() {
		parent::execute();
		
		// check permissions
		WCF::getUser()->checkPermission('admin.portal.customBox.canExport');
		
		$boxes = array();
		// single box export
		if ($this->box) {
			if ($this->box->className == 'CustomBox') {
				$boxes[] = $this->box;
			}
			else throw new IllegalLinkException();
		}
		else {
			$boxes = PortalBox::loadBoxes();
			foreach ($boxes as $key => $box) {
				if ($box->className != 'CustomBox') {
					unset($boxes[$key]);
				}
			}
		}
		
		if (!count($boxes)) throw new IllegalLinkException();

		// XML header
		$xmlString = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n";
		$xmlString .= "<customBoxes>\n";
		foreach ($boxes as $box) {
			$box->initialize();
			$xmlString .= "\t<box>\n";
			$xmlString .= "\t\t<title><![CDATA[".StringUtil::escapeCDATA((CHARSET != 'UTF-8' ? StringUtil::convertEncoding(CHARSET, 'UTF-8', $box->cacheData['title']) : $box->cacheData['title']))."]]></title>\n";
			$xmlString .= "\t\t<content><![CDATA[".StringUtil::escapeCDATA((CHARSET != 'UTF-8' ? StringUtil::convertEncoding(CHARSET, 'UTF-8', $box->cacheData['content']) : $box->cacheData['content']))."]]></content>\n";
			$xmlString .= "\t\t<parserType><![CDATA[".StringUtil::escapeCDATA((CHARSET != 'UTF-8' ? StringUtil::convertEncoding(CHARSET, 'UTF-8', $box->cacheData['parserType']) : $box->cacheData['parserType']))."]]></parserType>\n";
			$xmlString .= "\t\t<boxType><![CDATA[".StringUtil::escapeCDATA((CHARSET != 'UTF-8' ? StringUtil::convertEncoding(CHARSET, 'UTF-8', $box->boxType) : $box->boxType))."]]></boxType>\n";
			$xmlString .= "\t\t<forceActive><![CDATA[".StringUtil::escapeCDATA((CHARSET != 'UTF-8' ? StringUtil::convertEncoding(CHARSET, 'UTF-8', $box->forceActive) : $box->forceActive))."]]></forceActive>\n";
			$xmlString .= "\t</box>\n";
		}
		$xmlString .= "</customBoxes>";

		$fileName = TMP_DIR.'customBoxes.xml';
		$file = new File($fileName, 'wb');
		$file->write($xmlString);
		$file->close();
		@$file->chmod(0777);
		
		// export language variables

		$itemNames = array();
		foreach ($boxes as $box) {
			$itemNames[] = $box->cacheData['title'];
		}
		$languageItems = array();

		$sql = "SELECT		languageItem, languageItemValue, languageCategory, languageCode
			FROM		wcf".WCF_N."_package_dependency package_dependency,
					wcf".WCF_N."_language_item language_item
			LEFT JOIN	wcf".WCF_N."_language_category language_category
			ON		(language_category.languageCategoryID = language_item.languageCategoryID)
			LEFT JOIN	wcf".WCF_N."_language language
			ON		(language.languageID = language_item.languageID)
			WHERE 		language_item.packageID = package_dependency.dependency
					AND	languageItem IN ('".implode("','", array_map('escapeString', $itemNames))."')				
					AND package_dependency.packageID = ".PACKAGE_ID."
			ORDER BY 	package_dependency.priority ASC";
		$result = WCF::getDB()->sendQuery($sql);
		while ($row = WCF::getDB()->fetchArray($result)) {
			$languageItems[$row['languageCode']][$row['languageCategory']][$row['languageItem']] = $row['languageItemValue'];
		}

		$languageFiles = array();

		foreach ($languageItems as $languageCode => $categories) {
			$outputString = '';

			// bom
			$outputString .= "\xEF\xBB\xBF";

			// header
			$outputString .= "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n<!DOCTYPE language SYSTEM \"http://www.woltlab.com/DTDs/language.dtd\">\n<language languagecode=\"".$languageCode."\">\n";
				
			ksort($categories);
				
			foreach ($categories as $category => $categoryItems) {
				// sort items
				ksort($categoryItems);
					
				// category header
				$outputString .= "\t<category name=\"".$category."\">\n";
					
				// items
				foreach ($categoryItems as $item => $value) {
					if (CHARSET != 'UTF-8') {
						$value = StringUtil::convertEncoding(CHARSET, 'UTF-8', $value);
					}
					$outputString .= "\t\t<item name=\"".$item."\"><![CDATA[".StringUtil::escapeCDATA($value)."]]></item>\n";
				}
					
				// category footer
				$outputString .= "\t</category>\n";
			}
				
			$outputString .= "</language>";
						
			$languageFileName = TMP_DIR.$languageCode.'.xml';
			$file = new File($languageFileName, 'wb');
			$file->write($outputString);
			$file->close();
			@$file->chmod(0777);
			
			$languageFiles[] = $languageFileName;
		}
		
		$tarFileName = FileUtil::getTemporaryFilename('tmpFile_', '.tar');
		$tar = new TarWriter($tarFileName);
		array_unshift($languageFiles, $fileName);
		$tar->add($languageFiles, '', TMP_DIR);
		$tar->create();

		// file type
		header('Content-Type: application/octet-stream');

		// file name
		header('Content-Disposition: attachment; filename="customBoxesExport_'.date('Y_m_d_H_i').'.tar"');

		// send file size
		header('Content-Length: '.filesize($tarFileName));
			
		// no cache headers
		header('Pragma: no-cache');
		header('Expires: 0');
			
		// send file
		readfile($tarFileName);
		
		// cleanup
		@unlink($fileName);
		@unlink($tarFileName);
		foreach ($languageFiles as $file) {
			@unlink($file);
		}
	}
}
?>