<?php
// wcf imports
require_once(WCF_DIR.'lib/acp/form/ACPForm.class.php');
require_once(WCF_DIR.'lib/system/io/Tar.class.php');

// wbb imports
require_once(WBB_DIR.'lib/data/boxes/CustomBoxEditor.class.php');

/**
 * A form for importing custom portal boxes
 *
 * @package		com.woltlab.community.portal
 * @author		WoltLab Community
 * @copyright	2009 WoltLab Community
 * @license		GNU Lesser Public License <http://www.gnu.org/licenses/lgpl.html>
 * @subpackage	acp.form
 * @category	Portal
 */
class CustomBoxImportForm extends ACPForm {
	public $neededPermissions ='admin.portal.customBox.canImport';
	public $templateName = 'customBoxImport';
	public $action = 'add';
	public $activeMenuItem = 'wbb.acp.menu.link.content.portal.customBoxes.import';
	
	/**
	 * The uploaded import file
	 * 
	 * @var ressource
	 */
	public $file;	

	/**
	 * An array containing the data of all boxes which are about to be imported
	 * 
	 * @var array<mixed>
	 */
	public $boxData = array();
	
	/**
	 * @see Form::readFormParameters()
	 */
	public function readFormParameters() {
		parent::readFormParameters();
		
		if (isset($_FILES['file'])) $this->file = $_FILES['file'];		
	} 		
	
	/**
	 * @see Form::validate()
	 */
	public function validate() {
		parent::validate();

		// upload
		if ($this->file && $this->file['error'] != 4) {
			if ($this->file['error'] != 0) {
				throw new UserInputException('file', 'uploadFailed');
			}		
		
			$fileName = $_FILES['file']['tmp_name'];			
			$newName = TMP_DIR.$_FILES['file']['name'];			
			
			if (!move_uploaded_file($fileName, $newName)) {
				throw new UserInputException('file');	
			}			
			$fileName = $newName;
			$tar = new Tar($fileName);						
			
			// start handling the box data
			$xml = new XML();
			try {
				$xml->loadString($tar->extractToString('customBoxes.xml'));
			}
			catch (Exception $e) { // bugfix to avoid file caching problems
				$xml->loadString($tar->extractToString('customBoxes.xml'));
			}
			$tree = $xml->getElementTree('box');			
			foreach ($tree['children'] as $box) {
				$data = array();
				foreach ($box['children'] as $variables) {					
					$data[$variables['name']] = $variables['cdata'];																					
				}
				$this->boxData[] = $data;
			}
			
			// get languages
			foreach ($tar->getContentList() as $index => $file) {
				$languageXML = new XML();
				if ($file['filename'] == 'customBoxes.xml' || StringUtil::indexOf($file['filename'], '.xml') == -1) {
					continue;
				}
				try {
					$languageXML->loadString($tar->extractToString($file['filename']));
				}
				catch (Exception $e) { // bugfix to avoid file caching problems
					$languageXML->loadString($tar->extractToString($file['filename']));
				}
				
				$languageCode = LanguageEditor::readLanguageCodeFromXML($languageXML);				
				
				$languageEditor = LanguageEditor::getLanguageByCode($languageCode);
				
				if ($languageEditor === null) {
					continue;
				}
				
				try {
					$languageEditor->updateFromXML($languageXML, PORTAL_PACKAGE_ID);
				}
				catch (Exception $e) {}
			}
		}
	}
	
	/**
	 * @see Form::save()	 
	 */
	public function save() {
		parent::save();
		
		foreach ($this->boxData as $box) {
			CustomBoxEditor::create($box['title'], $box['content'], $box['boxType'], $box['parserType'], $box['forceActive'], true);
		}
		$this->saved();				
		
		WCF::getTPL()->assign('success', true);
	}
	
	/**
	 * @see Page::show()
	 */
	public function show() {
		// check master password
		WCFACP::checkMasterPassword();
		
		parent::show();
	}
}
?>