<?php
// wcf imports
require_once(WCF_DIR.'lib/page/SortablePage.class.php');

// wbb imports
require_once(WBB_DIR.'lib/data/boxes/PortalBox.class.php');

/**
 * A page listing all custom boxes
 *
 * @package		com.woltlab.community.portal
 * @author		WoltLab Community
 * @copyright	2009 WoltLab Community
 * @license		GNU Lesser Public License <http://www.gnu.org/licenses/lgpl.html>
 * @subpackage	acp.page
 * @category	Portal
 */
class CustomBoxListPage extends SortablePage {
	public $neededPermissions ='admin.portal.customBox.canAdd';	
	public $templateName = 'customBoxList';
	public $defaultSortField = 'customBoxID';	
	
	public $boxes = array();
	public $deletedBoxID = 0;
	
	/**
	 * @see Page::readParameters()	 
	 */
	public function readParameters() {
		parent::readParameters();
		
		if (isset($_POST['deletedBoxID'])) $this->deletedBoxID = intval($_POST['deletedBoxID']);
	}
	
	/**
	 * @see SortablePage::validateSortField()	 
	 */
	public function validateSortField() {
		parent::validateSortField();
		
		switch ($this->sortField) {
			case 'boxID':
			case 'title': 
			case 'parserType':
			case 'boxType':
			case 'forceActive':
				break;
			default:
				$this->sortField = $this->defaultSortField;
		}
	}
	
	/**
	 * @see Page::readData()	 
	 */
	public function readData() {
		parent::readData();
		
		$this->readBoxes();
	}
	
	/**
	 * Reads all boxes
	 * 
	 * @return void	 
	 */
	protected function readBoxes() {
		$sql = "SELECT		customBox.*, box.boxType 
			FROM 		wbb".WBB_N."_portalboxes_custom customBox
			LEFT JOIN	wbb".WBB_N."_portalboxes box
			ON 		(box.boxID = customBox.boxID)
			ORDER BY	".$this->sortField." ".$this->sortOrder;
		$result = WCF::getDB()->sendQuery($sql, $this->itemsPerPage, ($this->pageNo - 1) * $this->itemsPerPage);
		
		while ($row = WCF::getDB()->fetchArray($result)) {
			$this->boxes[] = new PortalBox(null, $row);
		}
	}
	
	/**
	 * @see Page::assignVariables()	 
	 */
	public function assignVariables() {
		parent::assignVariables();
		
		WCF::getTPL()->assign(array(
			'boxes' => $this->boxes,
			'deletedBoxID' => $this->deletedBoxID
		));
	}
	
	/**
	 * @see Page::show()	 
	 */
	public function show() {
		WCFACP::getMenu()->setActiveMenuItem('wbb.acp.menu.link.content.portal.customBoxes.list');
		
		parent::show();				
	}
	
	/**
	 * @see MultipleLinkPage::countItems()
	 * @return integer
	 */
	public function countItems() {
		parent::countItems();
		
		$sql = "SELECT	COUNT(customBoxID) AS count
			FROM	wbb".WBB_N."_portalboxes_custom";
		$row = WCF::getDB()->getFirstRow($sql);
		
		return $row['count'];
	}
}
?>