<?php
// wcf imports
require_once(WCF_DIR.'lib/page/AbstractPage.class.php');
require_once(WCF_DIR.'lib/acp/package/update/PackageUpdate.class.php');

/**
 * Shows the list of available portal boxes
 *
 * @package		com.woltlab.community.portal
 * @author		WoltLab Community
 * @copyright	2009 WoltLab Community
 * @license		GNU Lesser Public License <http://www.gnu.org/licenses/lgpl.html>
 * @subpackage	acp.page
 * @category	Portal
 */
class PortalBoxesPackageListPage extends AbstractPage {	
	public $neededPermissions = 'admin.system.package.canInstallPackage';
	
	public $portalIdentifier = 'com.woltlab.community.portal';	

	/**
	 * @see	Page::readData()
	 */
	public function readData() {
		parent::readParameters();

		// refresh package database
		PackageUpdate::refreshPackageDatabase();		

		// get portal box packages
		$this->searchPortalBoxPackages();
	}

	/**
	 * Searches all available package servers for portal boxes
	 * 
	 * @return void
	 */
	protected function searchPortalBoxPackages() {
		$boxes = '0';

		// get portal version
		$sql = "SELECT packageVersion FROM wcf".WCF_N."_package
					WHERE packageID = ".PORTAL_PACKAGE_ID;
		$row = WCF::getDB()->getFirstRow($sql);
		$portalVersion = $row['packageVersion'];

		// get update server data
		$updateServers = UpdateServer::getActiveUpdateServers();
		$packageUpdateServerIDs = implode(',', array_keys($updateServers));
		if (empty($packageUpdateServerIDs)) return $boxes;

		// get all update versions
		$sql = "SELECT		pu.*,
					puv.packageUpdateVersionID, puv.updateType, puv.timestamp, puv.file, puv.packageVersion,
					pur.package AS requirement, pur.minversion AS requirementVersion
			FROM		wcf".WCF_N."_package_update pu
			LEFT JOIN	wcf".WCF_N."_package_update_version puv			
			ON		(puv.packageUpdateID = pu.packageUpdateID)
			LEFT JOIN	wcf".WCF_N."_package_update_requirement pur
			ON		(pur.packageUpdateVersionID = puv.packageUpdateVersionID)
			WHERE		pu.packageUpdateServerID IN (".$packageUpdateServerIDs.")
			AND			puv.updateType = 'install'
			AND			pur.packageUpdateVersionID IN (
												SELECT packageUpdateVersionID
												FROM wcf".WCF_N."_package_update_requirement
												WHERE package = '".$this->portalIdentifier."'
			)";
		$result = WCF::getDB()->sendQuery($sql);		
		while ($row = WCF::getDB()->fetchArray($result)) {
			// check if this box is already installed
			if (!WCF::getPackageID($row['package'])) {
				if ($row['requirement'] == $this->portalIdentifier) {
					if (empty($row['requirementVersion']) || Package::compareVersion($portalVersion, $row['requirementVersion']) != -1) {
						if (!empty($boxes)) $boxes .= ',';
						$boxes .= $row['packageUpdateID']; 
					}
				}				
			}
		}
		
		// save search
		$sql = "INSERT INTO	wcf".WCF_N."_search
					(userID, searchData, searchDate, searchType)
			VALUES		(".WCF::getUser()->userID.", '".$boxes."', ".TIME_NOW.", 'packages')";
		WCF::getDB()->sendQuery($sql);
		$searchID = WCF::getDB()->getInsertID();
		
		// forward
		HeaderUtil::redirect('index.php?page=PackageUpdateSearchResult&searchID='.$searchID.'&packageID='.PACKAGE_ID.SID_ARG_2ND_NOT_ENCODED);				
	}
	
	/**
	 * @see Page::show()
	 */
	public function show() {
		// set active menu item
		WCFACP::getMenu()->setActiveMenuItem('wcf.acp.menu.link.package');

		// check master password
		WCFACP::checkMasterPassword();

		parent::show();
	}
}
?>