<?php
//Workarround for PHP Strict Standards warnings.
error_reporting(0);

// wbb imports
require_once(WBB_DIR.'lib/data/boxes/PortalBoxEditor.class.php');
require_once(WBB_DIR.'lib/data/boxes/PortalHelpers.class.php');

// wcf imports
require_once(WCF_DIR.'lib/data/user/option/UserOptionEditor.class.php');
require_once(WCF_DIR.'lib/system/language/LanguageEditor.class.php');
require_once(WCF_DIR.'lib/util/XML.class.php');
require_once(WCF_DIR.'lib/util/pip/LanguagesXMLPIP.class.php');

/**
 * Provides functions to create, update and delete custom boxes
 *
 * @package	com.woltlab.community.portal
 * @author	Alexander Ebert, Oliver Kliebisch, Martin Schwendowius
 * @copyright	2009 - 2012 wbbaddons.de
 * @license	GNU Lesser General Public License <http://www.gnu.org/licenses/lgpl.html>
 * @subpackage	data.boxes
 * @category	Portal
 */
class CustomBoxEditor extends PortalBoxEditor {

	/**
	 * Creates a new custom box
	 *
	 * @param	string	$title
	 * @param 	string	$content
	 * @param 	integer	$boxType 1 = center, 2 = left, 3 = right
	 * @param	string	$parserType The way how the content should be treated
	 * @param	integer	$forceActive Forces a box to stay open ignoring user settings
	 * @param	integer	$languageID specific language ID used for box title
	 * @param 	integer	$packageID Defaults to the packageID of the portal
	 * @return	CustomBoxEditor	A custom box editor object of the created box
	 */
	public static function create($title, $content, $boxType = 2, $parserType = 'plain', $forceActive = 0, $import = false, $packageID = PORTAL_PACKAGE_ID) {

		$boxTitle = '';

		// save custom box data
		$sql = "INSERT INTO	wbb".WBB_N."_portalboxes_custom
					(title, content, parserType)
			VALUES		('".escapeString($boxTitle)."',
					'".escapeString($content)."',
					'".$parserType."')";
		WCF::getDB()->sendQuery($sql);

		$customBoxID = WCF::getDB()->getInsertID();

		// create the portal box
		$portalBox = parent::create('customBox'.$customBoxID, $boxType, 'CustomBox', 1, 'customBox', 'CustomBox', 0, 1, 0, 0, 0, '', '', $packageID);


		$boxTitle = 'wbb.portal.box.customBox'.$customBoxID.'.title';

		//set import title value
		if ($import) {
			WCF::getDB()->sendQuery("UPDATE wcf".WCF_N."_language_item SET languageItem = '".$boxTitle."' WHERE languageItem = '".$title."'");
			/*
			The content of the following two rows was a simple "$title = WCF::getLanguage()->get($boxTitle);" before,
			but i have to fire this unnecessary database query because the damn fucking WCF Language classes are sucking in the
			second loop of the import form and does not deliver any more language content. Sure Woltlab, it is "WORKING AS DESIGNED".
			*/
			$tmp = WCF::getDB()->getFirstRow("SELECT languageItemValue as item FROM wcf".WCF_N."_language_item WHERE languageItem = '".$boxTitle."'");
			$title = $tmp['item'];
		}

		// set box id to custom box
		$sql = "UPDATE wbb".WBB_N."_portalboxes_custom
					SET boxID = ".$portalBox->boxID.",
					title = '".escapeString($boxTitle)."'
					WHERE customBoxID = ".$customBoxID;
		WCF::getDB()->sendQuery($sql);

		// set force active and box permission
		$sql = "UPDATE wbb".WBB_N."_portalboxes
			SET	forceActive = ".intval($forceActive).",
				permissions = 'user.portal.customboxes.customBox".$customBoxID.".canView'
			WHERE	boxID = ".$portalBox->boxID;
		WCF::getDB()->sendQuery($sql);

		//generate permission
		self::generatePermission($customBoxID, $title, $import);

		//reset cache
		self::resetCache();
		$customBoxEditor = new CustomBoxEditor(null, null, $portalBox);

		switch ($parserType) {
			case 'template':
				$customBoxEditor->writeTemplate();
				break;
			case 'php':
				$customBoxEditor->writeScript();
				break;
		}

		return $customBoxEditor;
	}

	/**
	 * Updates the current box
	 *
	 * @param	string	$title
	 * @param 	string	$content
	 * @param 	integer	$boxType 1 = center, 2 = left, 3 = right
	 * @param	string	$parserType The way how the content should be treated
	 * @param	integer	$forceActive Forces a box to stay open ignoring user settings
	 * @param	integer	$languageID specific language ID used for box title and grouprights
	 * @return 	void
	 */
	public function update($title, $content, $boxType, $parserType, $forceActive, $languageID = 0) {
		//update custom box table
		$sql = "UPDATE	wbb".WBB_N."_portalboxes_custom
			SET	".($languageID == 0 ? "title = '".escapeString($title)."'," : '')."
				content = '".escapeString($content)."',
				parserType = '".escapeString($parserType)."'
			WHERE	boxID = ".$this->boxID;
		WCF::getDB()->sendQuery($sql);

		//update portalbox table
		parent::update(null, $boxType);

		//deletes template or script
		if ($this->cacheData['parserType'] != $parserType) {
			if ($this->cacheData['parserType'] == 'template') {
				$this->deleteTemplate();
			}
			else if ($this->cacheData['parserType'] == 'php') {
				$this->deleteScript();
			}
		}

		//rewrites template or script
		$this->cacheData['content'] = $content;
		switch ($parserType) {
			case 'template':
				$this->writeTemplate();
				break;
			case 'php':
				$this->writeScript();
				break;
		}
		//updates the language files
		if ($languageID != 0) {
			$customID = WCF::getDB()->getFirstRow("SELECT customBoxID as customBoxID FROM wbb".WBB_N."_portalboxes_custom WHERE boxID = ".$this->boxID);
			$customBoxID = intval($customID['customBoxID']);
			$import = false;
			self::generateLanguageItems($customBoxID, $title, $import, $languageID);
		}

		//update force active value
		$sql = "UPDATE	wbb".WBB_N."_portalboxes
			SET	forceActive = ".intval($forceActive)."
			WHERE	boxID = ".$this->boxID;
		WCF::getDB()->sendQuery($sql);

		//reset cache
		self::resetCache();
	}

	/**
	 * Deletes the current box and its parameters
	 *
	 * @return void
	 */
	public function delete() {
		//getCustomBoxID
		$customID = WCF::getDB()->getFirstRow("SELECT customBoxID as customBoxID FROM wbb".WBB_N."_portalboxes_custom WHERE boxID = ".$this->boxID);
		$customBoxID = intval($customID['customBoxID']);

		//delete permissions
		self::deletePermissions($customBoxID);
		self::deleteLanguageItems($customBoxID);

		//delete box from custom box table
		$sql = "DELETE FROM wbb".WBB_N."_portalboxes_custom
					WHERE boxID = ".$this->boxID;
		WCF::getDB()->sendQuery($sql);

		//delete template or script
		switch ($this->cacheData['parserType']) {
			case 'template':
				$this->deleteTemplate();
				break;
			case 'php':
				$this->deleteScript();
				break;
		}

		//delete portalbox from portalbox table
		parent::delete();

		//reset cache
		self::resetCache();
	}

	/**
	 * Writes a template file with the content of this box
	 *
	 * @return void
	 */
	public function writeTemplate() {
		$this->initialize();

		require_once(WCF_DIR.'lib/system/io/File.class.php');
		$file = new File(WBB_DIR.'templates/customBox'.$this->boxID.'.tpl');
		// write template
		$file->write($this->cacheData['content']);
		@$file->chmod(0777);
	}

	/**
	 * Stores the php code in a script file
	 *
	 * @return void
	 */
	public function writeScript() {
		$this->initialize();

		require_once(WCF_DIR.'lib/system/io/File.class.php');
		$file = new File(WBB_DIR.'lib/data/boxes/scripts/customBox'.$this->boxID.'.php');
		// write script
		$file->write($this->cacheData['content']);
		@$file->chmod(0777);
	}

	/**
	 * Deletes the template of this box
	 *
	 * @return void
	 */
	public function deleteTemplate() {
		//delete template
		$file = WBB_DIR.'templates/customBox'.$this->boxID.'.tpl';
		@unlink($file);
	}

	/**
	 * Deletes the PHP script of this box
	 *
	 * @return void
	 */
	public function deleteScript() {
		//delete php file
		$file = WBB_DIR.'lib/data/boxes/scripts/customBox'.$this->boxID.'.php';
		@unlink($file);
	}

	/**
	 * Clears the custom box, portal box cache, WCF cache and resets Sessions
	 *
	 * @see		PortalBox::resetCache()
	 * @param 	boolean	$clearDataCache Clear the data cache of all boxes
	 * @return 	void
	 */
	public static function resetCache($clearDataCache = false) {
		//clear portalbox cache
		parent::resetCache($clearDataCache);

		//clear custom box cache
		WCF::getCache()->clear(WBB_DIR.'cache/', 'cache.customBoxes.php', true);

		// clear WCF cache
		WCF::getCache()->clear(WCF_DIR.'cache', '*.php', true);

		// update sessions
                require_once(WCF_DIR.'lib/system/session/Session.class.php');
                Session::resetSessions();
	}


	/**
	 * Generates the CustomBox Permissions
	 *
	 * @return void
	 */
	protected function generatePermission($boxID, $boxTitle, $import) {
		$categoryName;

		//get all available groups
		require_once(WCF_DIR.'lib/data/user/group/Group.class.php');
		$groupIDArray = Group::getAllGroups();
		foreach ($groupIDArray as $groupID => $uselessCrap) {
			$groupIDs[] = $groupID;
		}

		//build permission variables
		$categoryName = "user.portal.customboxes";
		$permission = $categoryName.".customBox".$boxID.".canView";

		//write group option
		$tmp = WCF::getDB()->getFirstRow("SELECT MAX(showOrder) AS showorder FROM wcf".WCF_N."_group_option WHERE categoryName='".escapeString($categoryName)."'");
		$showorder = $tmp['showorder']+1;

		$sql="INSERT INTO wcf".WCF_N."_group_option (packageID, optionName, categoryName, optionType, defaultValue, validationPattern, enableOptions, showOrder)
			VALUES(".PORTAL_PACKAGE_ID.", '".$permission."', '".$categoryName."', 'boolean', '1', '', '', ".$showorder.")";
		WCF::getDB()->sendQuery($sql);

		//write group option values
		foreach ($groupIDs as $groupID) {
			$sql = "SELECT 	optionID AS optionID 
				FROM 	wcf".WCF_N."_group_option 
				WHERE 		optionName='".escapeString($permission)."'
					AND	packageID = ".PORTAL_PACKAGE_ID;
			$tmp = WCF::getDB()->getFirstRow($sql);
			$optionID = $tmp['optionID'];
			$sql="INSERT INTO wcf".WCF_N."_group_option_value (groupID, optionID, optionValue) VALUES (".$groupID.", ".$optionID.", '1')";
			WCF::getDB()->sendQuery($sql);
		}

		//generate language Items
		self::generateLanguageItems($boxID, $boxTitle, $import);
	}

	/**
	 * Deletes permissions
	 *
	 * @return void
	 */
	protected function deletePermissions($boxID) {

		//select matching optionID
		$tmp = WCF::getDB()->getFirstRow("SELECT optionID AS optionID FROM wcf".WCF_N."_group_option WHERE optionName = 'user.portal.customboxes.customBox".$boxID.".canView'");
		$optionID = $tmp['optionID'];

		//delete optionValues
		$sql = "DELETE FROM   wcf".WCF_N."_group_option_value WHERE optionID = ".$optionID;
        	WCF::getDB()->sendQuery($sql);

        	//delete groupOption
        	$sql = "DELETE FROM   wcf".WCF_N."_group_option
        	WHERE   optionName = 'user.portal.customboxes.customBox".$boxID.".canView'";
        	WCF::getDB()->sendQuery($sql);
	}

	/**
	 * Generates or updates the language items
	 *
	 * @param	integer	$boxID
	 * @param	string	$boxTitle
	 * @param	integer	$languageID
	 */
	protected function generateLanguageItems($boxID, $boxTitle, $import, $languageID = 0) {
		$boxID = intval($boxID);
		$boxTitle = escapeString($boxTitle);
		$languageData = array();

		//get available languages
		$languageCodes = Language::getLanguageCodes();

		// get default values
		$sql = "SELECT	*
			FROM	wcf".WCF_N."_language_item
			WHERE	languageItem IN(
					'wcf.acp.group.portal.boxes.customBox.canView',
					'wcf.acp.group.portal.boxes.customBox.canView.description'
				)
			AND	packageID = ".PORTAL_PACKAGE_ID;
		if ($languageID) $sql .= " AND languageID = ".intval($languageID);
		$result = WCF::getDB()->sendQuery($sql);

		// read default labels
		while ($row = WCF::getDB()->fetchArray($result)) {
			// skip unknown languages
			if (!array_key_exists($row['languageID'], $languageCodes)) continue;

			// create language item name and value
			$name = str_replace('wcf.acp.group.portal.boxes.customBox', 'option.user.portal.customboxes.customBox'.$boxID, $row['languageItem']);
			$value = str_replace('{$boxname}', $boxTitle, $row['languageItemValue']);

			// build entire language item
			$languageData[][$languageCodes[$row['languageID']]]['wcf.acp.group'][$name] = PortalHelpers::convertEncoding($value);
			if (!$import) $languageData[][$languageCodes[$row['languageID']]]['wbb.portal']['box.customBox'.$boxID.'.title'] = PortalHelpers::convertEncoding($boxTitle);
		}

		// create xml string
		foreach ($languageData as $language) {
			//create XML string
			$xml = LanguagesXMLPIP::create($language, true);

			// parse xml
			$xmlObj = new XML();
			$xmlObj->loadString($xml);

			// import language xml
			LanguageEditor::importFromXML($xmlObj, PORTAL_PACKAGE_ID);
		}
	}

	/**
	 * Deletes permissions language items
	 */
	protected function deleteLanguageItems($boxID) {
		// delete items
        	$sql = "DELETE FROM   wcf".WCF_N."_language_item
        		WHERE   languageItem LIKE 'wcf.acp.group.option.user.portal.customboxes.customBox".$boxID.".canView'
        		OR languageItem LIKE 'wcf.acp.group.option.user.portal.customboxes.customBox".$boxID.".canView.description'
			OR languageItem LIKE 'wbb.portal.box.customBox".$boxID.".title'";
        	WCF::getDB()->sendQuery($sql);

        	//clear language cache files
        	LanguageEditor::deleteLanguageFiles();
        }
}
?>