<?php
// wbb imports
require_once(WBB_DIR.'lib/data/boxes/PortalBox.class.php');

/**
 * Provides functions to edit a portal box
 * 
 * @package	com.woltlab.community.portal
 * @author	Oliver Kliebisch
 * @copyright	2009 WoltLab Community
 * @license	GNU Lesser Public License <http://www.gnu.org/licenses/lgpl.html>
 * @subpackage	data.boxes
 * @category	Portal
 */
class PortalBoxEditor extends PortalBox {		
	/**
	 * Creates a new PortalBoxEditor object.
	 * @see PortalBox::__construct()
	 */
	public function __construct($boxID, $row = null, $cacheObject = null, $useCache = true) {
		if ($useCache) parent::__construct($boxID, $row, $cacheObject);
		else {
			$sql = "SELECT	*
				FROM	wbb".WBB_N."_portalboxes
				WHERE	boxID = ".$boxID;
			$row = WCF::getDB()->getFirstRow($sql);
			$row['cacheTypeClassFile'] = WBB_DIR.'lib/system/portalbox/cachebuilder/'.ucfirst($row['cacheType']).'CacheBuilder.class.php';
			$row['cacheTypeClassName'] = StringUtil::getClassName($row['cacheTypeClassFile']);
			$row['boxClassFile'] = WBB_DIR.'lib/data/boxes/'.$row['className'].'.class.php';
			parent::__construct(null, $row);
		}
	}
	
	/**
	 * Creates a new portal box
	 * 	 
	 * @param 	string	$boxName 				The name of the box
	 * @param 	integer	$boxType				The boxType (=its position)
	 * @param 	string	$className				The box classname
	 * @param 	integer	$useCache				1 if this box uses caching
	 * @param 	string	$cacheType				The cache type
	 * @param 	string	$cacheBuilderClass		The cache class name
	 * @param 	integer	$sortOrder				The box sortOrder
	 * @param 	integer	$boxActivation			1 if this box is enabled
	 * @param 	integer	$minLifeTime			The minimum life time of the box cache
	 * @param 	integer	$maxLifeTime			The maximum life time of the box cache
	 * @param 	integer	$showInForum			1 if this box should be displayed in the forum
	 * @param 	string	$permissions			The needed permission to view this box
	 * @param 	string	$options				The needed active options to view this box
	 * @param 	integer	$packageID				The packageID this box belongs to
	 * @param 	array	$additionalFields		An array of additional fields
	 * @return 	PortalBoxEditor
	 */
	public static function create($boxName, $boxType, $className, $useCache = 0, $cacheType = '', $cacheBuilderClass = '', $sortOrder = 0, $boxActivation = 1, $minLifeTime = 0, $maxLifeTime = 0, $showInForum = 0, $permissions = '', $options = '', $packageID = PACKAGE_ID, $additionalFields = array()) {
		// handle sort order
		if ($sortOrder == 0) {
			$sql = "SELECT MAX(sortOrder) AS sortOrder 
				FROM wbb".WBB_N."_portalboxes
				WHERE boxType = ".$boxType;
			$row = WCF::getDB()->getFirstRow($sql);
			if (!empty($row)) $sortOrder = intval($row['sortOrder']) + 1;
			else $sortOrder = 1;	
		}
		else {
			$sql = "UPDATE wbb".WBB_N."_portalboxes
						SET sortOrder = sortOrder + 1
						WHERE boxType = ".$boxType."
						AND sortOrder >= ".$sortOrder;
			WCF::getDB()->sendQuery($sql);
		}
		// save data
		$boxID = self::insert($boxName, array_merge($additionalFields, array(
			'boxType' => $boxType,			
			'className' => $className,
			'useCache' => $useCache,
			'cacheType' => $cacheType,
			'cacheBuilderClass' => $cacheBuilderClass,
			'sortOrder' => $sortOrder,
			'boxActivation' => $boxActivation,
			'minLifeTime' => $minLifeTime,
			'maxLifeTime' => $maxLifeTime,
			'showInForum' => $showInForum,
			'permissions' => $permissions,
			'options' => $options,
			'packageID' => $packageID
		)));
		
		// get box
		$box = new PortalBoxEditor($boxID, null, null, false);

		self::createStatusUserOption($boxName.$boxID);
		
		return $box;
	}
	
	/**
	 * Creates the user option for a portal box
	 * 
	 * @param 	string	$name 		The name of the variable
	 * @param 	integer	$packageID	The packageID of the variable
	 * @return 	void
	 */
	protected static function createStatusUserOption($name, $packageID = PORTAL_PACKAGE_ID) {
		if (WCF::getUser()->getUserOption($name) === null) {
			// create status user option
			UserOptionEditor::create($name, 'hidden', 'boolean', 1, '', '', '', 0, 0, 4, 4, '', 0, 0, 0, $packageID);
			WCF::getCache()->clear(WCF_DIR . 'cache', 'cache.user-option-*');
		}		
	}
	
	/**
	 * Creates the box row in database table.
	 *
	 * @param 	string 		$boxName
	 * @param 	array		$additionalFields
	 * @return	integer		new box id
	 */
	public static function insert($boxName, $additionalFields = array()) { 
		$keys = $values = '';
		foreach ($additionalFields as $key => $value) {
			$keys .= ','.$key;
			if (is_int($value)) $values .= ",".$value;
			else $values .= ",'".escapeString($value)."'";
		}
		
		$sql = "INSERT INTO	wbb".WBB_N."_portalboxes
					(boxName
					".$keys.")
			VALUES		('".escapeString($boxName)."'
					".$values.")";
		WCF::getDB()->sendQuery($sql);
		return WCF::getDB()->getInsertID();
	}
	
	/**
	 * Updates the data of a box.
	 * 
	 * @param 	string	$boxName 				The name of the box
	 * @param 	integer	$boxType				The boxType (=its position)
	 * @param 	string	$className				The box classname
	 * @param 	integer	$useCache				1 if this box uses caching
	 * @param 	string	$cacheType				The cache type
	 * @param 	string	$cacheBuilderClass		The cache class name
	 * @param 	integer	$sortOrder				The box sortOrder
	 * @param 	integer	$boxActivation			1 if this box is enabled
	 * @param 	integer	$minLifeTime			The minimum life time of the box cache
	 * @param 	integer	$maxLifeTime			The maximum life time of the box cache
	 * @param 	integer	$showInForum			1 if this box should be displayed in the forum
	 * @param 	string	$permissions			The needed permission to view this box
	 * @param 	string	$options				The needed active options to view this box
	 * @param 	integer	$packageID				The packageID this box belongs to
	 * @param 	array	$additionalFields		An array of additional fields
	 * @return	void
	 */
	public function update($boxName = null, $boxType = null, $className = null, $useCache = null, $cacheType = null, $cacheBuilderClass = null, $sortOrder = null, $boxActivation = null, $minLifeTime = null, $maxLifeTime = null, $showInForum = null, $permissions = null, $options = null, $packageID = null, $additionalFields = array()) {
		$fields = array();
		if ($boxName !== null) $fields['boxName'] = $boxName;
		if ($boxType !== null) $fields['boxType'] = $boxType;
		if ($className !== null) $fields['classname'] = $className;
		if ($useCache !== null) $fields['useCache'] = $useCache;
		if ($cacheType !== null) $fields['cacheType'] = $cacheType;
		if ($cacheBuilderClass !== null) $fields['cacheBuilderClass'] = $cacheBuilderClass;
		if ($sortOrder !== null) $fields['sortOrder'] = $sortOrder;
		if ($boxActivation !== null) $fields['boxActivation'] = $boxActivation;
		if ($minLifeTime !== null) $fields['minLifeTime'] = $minLifeTime;
		if ($maxLifeTime !== null) $fields['maxLifeTime'] = $maxLifeTime;
		if ($showInForum !== null) $fields['showInForum'] = $showInForum;
		if ($permissions !== null) $fields['permissions'] = $permissions;
		if ($options !== null) $fields['options'] = $options;
		if ($packageID !== null) $fields['packageID'] = $packageID;		
		
		$this->updateData(array_merge($fields, $additionalFields));
	}
	
	/**
	 * Updates the data of a box.
	 *
	 * @param 	array		$fields
	 */
	public function updateData($fields = array()) { 
		$updates = '';
		foreach ($fields as $key => $value) {
			if (!empty($updates)) $updates .= ',';
			$updates .= $key.'=';
			if (is_int($value)) $updates .= $value;
			else $updates .= "'".escapeString($value)."'";
		}
		
		if (!empty($updates)) {
			$sql = "UPDATE	wbb".WBB_N."_portalboxes
				SET	".$updates."
				WHERE	boxID = ".$this->boxID;
			WCF::getDB()->sendQuery($sql);
		}
	}
	
	/**
	 * Deletes this box.
	 */
	public function delete() {
		$sql = "DELETE FROM wbb".WBB_N."_portalboxes
				WHERE boxID = ".$this->boxID;
		WCF::getDB()->sendQuery($sql);
		
		$userOption = new UserOptionEditor(WCF::getUser()->getUserOptionID($this->getStatusVariable()));	
		if ($userOption->optionID) {
			$userOption->delete();						
		}
		
		// delete cache
		WCF::getCache()->clear(WCF_DIR . 'cache', 'cache.user-option-*');
		
		self::resetCache();
	}
	
	/**
	 * Enables this portal box.	 
	 */
	public function enable() {
		$this->update(null, null, null, null, null, null, null, 1);		
	}
	
	/**
	 * Disables this portal box	  	 
	 */
	public function disable() {
		$this->update(null, null, null, null, null, null, null, 0);
	}
	
	/**
	 * Displays this portal box in the forum	 
	 */
	public function showBoxInForum() {
		$this->update(null, null, null, null, null, null, null, null, null, null, 1);
	}
	
	/**
	 * Hides this portal box in the forum	 
	 */
	public function hideBoxInForum() {
		$this->update(null, null, null, null, null, null, null, null, null, null, 0);
	}
	
	/**
	 * Moves this box
	 * 
	 * @param 	integer	$sortOrder
	 * @param 	integer	$boxType
	 * @see 	moveBox()
	 * @return 	void
	 */
	public function move($sortOrder, $boxType = null) {
		self::moveBox($this->boxID, $sortOrder, $boxType);
	}
	
	/**
	 * Moves a box
	 * 
	 * @param	integer	$boxID
	 * @param	integer	$sortOrder
	 * @param 	integer	$boxType
	 * @return void
	 */
	public static function moveBox($boxID, $sortOrder, $boxType = null) {
		$sql = "UPDATE wbb".WBB_N."_portalboxes
						SET sortOrder = ".$sortOrder.($boxType ? ", boxType = ".$boxType : "")."
						WHERE boxID = ".$boxID;		
		WCF::getDB()->sendQuery($sql);
	}
}
?>