<?php
// wcf imports
require_once(WCF_DIR.'lib/form/AbstractForm.class.php');
require_once(WCF_DIR.'lib/page/util/menu/PageMenu.class.php');
require_once(WCF_DIR.'lib/system/event/EventHandler.class.php');

// portal imports
require_once(WBB_DIR.'lib/data/boxes/PortalHelpers.class.php');
require_once(WBB_DIR.'lib/system/portalbox/PortalBoxFactory.class.php');

/**
 * Provides the portal sort form
 *
 * @package	com.woltlab.community.portal
 * @author	WoltLab Community
 * @copyright	2009-2010 WoltLab Community
 * @license	GNU Lesser Public License <http://www.gnu.org/licenses/lgpl.html>
 * @subpackage	acp.page
 * @category	Portal
 */
class PortalEditForm extends AbstractForm {
	public $neededPermissions = 'user.portal.general.canSortOwn';
	public $templateName = 'portalEdit';
 	public $portalVersionNumber = '2.2.2';

	protected static $portalBox = null;
	public $positions = array();
	public $boxTypes = array();
	public $activated = array();

	/**
	 * Determines current portal mode
	 *
	 * @var	string
	 */
	public $portalMode = 'edit';

	/**
	 * Initialize PortalHelper
	 */
	public function __construct() {
		PortalHelpers::init();

		parent::__construct();
	}

	/**
	 * @see	Form::readFormParameters()
	 */
	public function readFormParameters() {
		parent::readFormParameters();

		// read positions
		if (isset($_POST['positions']) && is_array($_POST['positions'])) {
			$this->positions = $_POST['positions'];
		}

		// read types
		if (isset($_POST['boxTypes']) && is_array($_POST['boxTypes'])) {
			$this->boxTypes = $_POST['boxTypes'];
		}

		// read activated
		if (isset($_POST['activated']) && is_array($_POST['activated'])) {
			$this->activated = $_POST['activated'];
		}
	}

	/**
	 * @see	Page::readData()
	 */
	public function readData() {
		parent::readData();

		// break if portal is disabled
		if (!MODULE_PORTAL) throw new IllegalLinkException();

		// break if portal is offline or user is guest
		if (PORTAL_OFFLINE || !WCF::getUser()->userID) throw new PermissionDeniedException();

		// register portal stylesheet
		PortalHelpers::addBoxStylesheet('portal', true);

		// initialize portal boxes
		self::getPortalBoxes();
	}

	/**
	 * @see	Form::save()
	 */
	public function save() {
		parent::save();

		// initialize column array
		$data = array(
			1 => array(),
			2 => array(),
			3 => array()
		);

		// assign boxes by column
		$i = 0;
		foreach ($this->positions as $boxID => $sortOrder) {
			$i++;
			if (array_key_exists($boxID, $this->boxTypes)) {
				$data[$this->boxTypes[$boxID]][$i] = $boxID;
			}
		}

		// sort boxes
		for ($i = 1; $i <= 3; $i++) {
			$tmp = array();
			ksort($data[$i], SORT_NUMERIC);

			// replace key (contains the sort order) with activation status
   			foreach ($data[$i] as $boxID) {
   				$tmp[$boxID] = (isset($this->activated[$boxID])) ? 1 : 0;
   			}

   			$data[$i] = $tmp;
		}

		// insert or update user sort
		$sql = "INSERT INTO	wbb".WBB_N."_portalboxes_sort_to_user
					(userID, leftboxes, centerboxes, rightboxes)
			VALUES		(".WCF::getUser()->userID.",
					'".serialize($data[2])."',
					'".serialize($data[1])."',
					'".serialize($data[3])."')
			ON DUPLICATE KEY UPDATE leftboxes=VALUES(leftboxes),
					centerboxes=VALUES(centerboxes),
					rightboxes=VALUES(rightboxes)";
		WCF::getDB()->sendQuery($sql);

		//update user table if necessary
		if (!WCF::getUser()->customPortalBoxSort) {
			$sql = "UPDATE wbb".WBB_N."_user SET customPortalBoxSort = 1 WHERE userID = ".WCF::getUser()->userID;
			WCF::getDB()->sendQuery($sql);
		}

		// reset user session
                require_once(WCF_DIR.'lib/system/session/Session.class.php');
                Session::resetSessions(WCF::getUser()->userID, true, false);

                //fire event saved
		$this->saved();

		//redirect to portal page
		HeaderUtil::redirect('index.php?page=Portal'.SID_ARG_2ND);

	}

	/**
	 * @see	Page::assignVariables()
	 */
	public function assignVariables() {
		parent::assignVariables();

		// append box-related stylesheets
		WCF::getTPL()->append('specialStyles', PortalHelpers::getBoxStylesheets());

		WCF::getTPL()->assign(array(
			'selfLink' => 'portal.php?'.SID_ARG_2ND, // deprecated?
			'portalboxes' => self::$portalBox,
			'portalMode' => $this->portalMode,
			'portalVersionNumber' => $this->portalVersionNumber,
			'allowSpidersToIndexThisPage' => false
		));
	}

	/**
	 * @see	Page::show()
	 */
	public function show() {
		// mark Portal as active
		PageMenu::setActiveMenuItem('wbb.header.menu.portal');

		parent::show();
	}

	/**
	 * Creates an unique instance of PortalBoxFactory
	 *
	 * @return	PortalBoxFactory
	 */
	public static function getPortalBoxes() {
		if (self::$portalBox === null) {
			self::$portalBox = new PortalBoxFactory(true, false, true, true, false, false, true);
		}

		return self::$portalBox;
	}
}
?>