<?php
// wcf imports
require_once(WCF_DIR.'lib/page/AbstractPage.class.php');
require_once(WCF_DIR.'lib/page/util/menu/PageMenu.class.php');

// portal imports
require_once(WBB_DIR.'lib/data/boxes/PortalHelpers.class.php');
require_once(WBB_DIR.'lib/system/portalbox/PortalBoxFactory.class.php');

/**
 * Loads and stores portal boxes
 *
 * @package	com.woltlab.community.portal
 * @author	WoltLab Community
 * @copyright	2009 WoltLab Community
 * @license	GNU Lesser Public License <http://www.gnu.org/licenses/lgpl.html>
 * @subpackage	page
 * @category	Portal
 */
class PortalPage extends AbstractPage {
	/**
	 * Needed permissions
	 *
	 * @var	string
	 */
	public $neededPermissions = 'user.portal.general.canView';

	/**
	 * Portal version
	 *
	 * @var	string
	 */
 	public $portalVersionNumber = '2.2.2';

	/**
	 * Template
	 *
	 * @var	string
	 */
	public $templateName = 'portal';

	/**
	 * Enables the left column and adds a margin to centerbox
	 *
	 * @var	boolean
	 */
	protected $leftColumnActive = true;

	/**
	 * Enables the right column and adds a margin to centerbox
	 *
	 * @var	boolean
	 */
	protected $rightColumnActive = true;

	/**
	 * Available portal boxes
	 */
	protected static $portalBox = null;
	
	/**
	 * Determines current portal mode
	 *
	 * @var	string
	 */
	public $portalMode = 'display';
	
	/**
	 * Show portal boxes
	 *
	 * @see	AbstractPage::assignVariables();
	 */
	public function assignVariables() {
		parent::assignVariables();

		// mark Portal as active
		PageMenu::setActiveMenuItem('wbb.header.menu.portal');

		// append box-related stylesheets
		WCF::getTPL()->append('specialStyles', PortalHelpers::getBoxStylesheets());

		WCF::getTPL()->assign(array(
			'selfLink' => 'portal.php?'.SID_ARG_2ND, // deprecated?
			'portalboxes' => self::$portalBox,
			'leftColumnActive' => $this->leftColumnActive,
			'rightColumnActive' => $this->rightColumnActive,
			'allowSpidersToIndexThisPage' => true,
			'portalMode' => $this->portalMode,
			'portalVersionNumber' => $this->portalVersionNumber
		));
	}

	/**
	 * Determine layout
	 *
	 * @see	AbstractPage::readData()
	 */
	public function readData() {
		parent::readData();

		// register portal stylesheet
		PortalHelpers::addBoxStylesheet('portal', true);

		// initialize portal boxes
		self::getPortalBoxes();

		// get column status
		$status = self::$portalBox->getColumnStatus();
		$this->leftColumnActive = $status['left'];
		$this->rightColumnActive = $status['right'];
	}

	/**
	 * Return portal boxes, intializes unique instance if neccessary
	 *
	 * @return	object
	 */
	public static function getPortalBoxes() {
		if (self::$portalBox === null) {
			self::$portalBox = new PortalBoxFactory(true, false, true, true, true);
		}

		return self::$portalBox;
	}
	
	public function show() {
		// pretend portal is not available
		if (!MODULE_PORTAL) {
			throw new IllegalLinkException();
		}
		
		// show offline message if applicable
		if (PORTAL_OFFLINE) {
			$this->templateName = 'portal_offline';
			WCF::getTPL()->assign(array(
				'offlinemessage' => PORTAL_OFFLINE_MESSAGE,
				'portalMode' => $this->portalMode,
				'portalVersionNumber' => $this->portalVersionNumber
			));
			
			// mark Portal as active
			PageMenu::setActiveMenuItem('wbb.header.menu.portal');
		}
		
		PortalHelpers::init();
		
		parent::show();
	}
}
?>
