<?php
// wbb imports
require_once(WBB_DIR.'lib/data/boxes/PortalHelpers.class.php');

// wcf imports
require_once(WCF_DIR.'lib/system/event/EventHandler.class.php');
require_once(WCF_DIR.'lib/system/event/EventListener.class.php');

/**
 * Displays portal boxes in forum
 *
 * @package	com.woltlab.community.portal
 * @author	WoltLab Community
 * @copyright	2009-2010 WoltLab Community
 * @license	GNU Lesser Public License <http://www.gnu.org/licenses/lgpl.html>
 * @subpackage	system.event.listener
 * @category	Portal
 */
class ShowBoxInBoardListener implements EventListener {
	protected $boxData = null;
	protected $executed = false;

	const PORTAL_COLUMN_NONE = 0;
	const PORTAL_COLUMN_LEFT = 1;
	const PORTAL_COLUMN_RIGHT = 2;
	const PORTAL_COLUMN_BOTH = 3;
	/**
	 * Insert portal boxes if applicable
	 *
	 * @see EventListener::execute()
	 * @return	void
	 */
	public function execute($eventObj, $className, $eventName) {
		
		// portal disabled
		if (MODULE_PORTAL == 0 || $this->executed) return;

		// omit boxes if inside portal
		if(isset($_GET['page']) && $_GET['page'] == 'Portal') return;
		
		// fix for typo in URL
		if(isset($_GET['page']) && $_GET['page'] == 'portal') return;

		// omit boxes if inside PortalEdit Form
		if(isset($_GET['form']) && $_GET['form'] == 'PortalEdit') return;

		// cannot see portal or disabled
		$columnStatus = (WCF::getUser()->userID) ? WCF::getUser()->enablePortalBoxesInBoard : PORTAL_BOXES_FORUM_ADMIN_DEFAULT;
		if ($columnStatus == self::PORTAL_COLUMN_NONE || WCF::getUser()->getPermission('user.portal.general.canView') == 0) return;

		// initialize PortalHelper
		PortalHelpers::init();

		// call execute event
		EventHandler::fireAction($this, 'execute');

		// load PortalBoxFactory for fetching and creating portal boxes
		require_once(WBB_DIR.'lib/system/portalbox/PortalBoxFactory.class.php');
		
		if (WCF::getUser()->userID) {
			$this->boxData = new PortalBoxFactory(true, true, true, true, true);
		} else {
			$this->boxData = new PortalBoxFactory(true, true, true, true);
		}
		$type2 = $this->boxData->getType2Data();
		$type3 = $this->boxData->getType3Data();

		// omit boxes if none enabled
		if(empty($type2) && empty($type3)) return;

		$leftColumn = array();
		$rightColumn = array();

		if ($columnStatus == self::PORTAL_COLUMN_LEFT || $columnStatus == self::PORTAL_COLUMN_BOTH) {
			// fill left column
			$leftColumn = $type2;
		}

		if ($columnStatus == self::PORTAL_COLUMN_RIGHT || $columnStatus == self::PORTAL_COLUMN_BOTH) {
			// fill right column
			$rightColumn = $type3;
		}

		// add portal css
		PortalHelpers::addBoxStylesheet('portal', true);

		// box data
		WCF::getTPL()->assign(array(
			'leftColumn' => $leftColumn,
			'rightColumn' => $rightColumn,
			'portalMode' => 'board'
		));

		//requirements for boxInBoard display
		WCF::getTPL()->append(array(
			'specialStyles' => PortalHelpers::getBoxStylesheets(),
			'userMessages' => WCF::getTPL()->fetch('portalHeader'),
			'additionalFooterContents' => WCF::getTPL()->fetch('portalFooter')
		));

		$this->executed = true;
	}
}
?>